@@ -1,6 +1,6 @@
 from dictshield.document import EmbeddedDocument
-from dictshield.base import UUIDField
-from dictshield.fields import StringField
+from dictshield.fields.base import UUIDField, StringField
+
 
 from brubeck.timekeeping import MillisecondField
 
