@@ -37,7 +37,7 @@ class Request(object):
         self.headers = headers
         self.body = body
 
-        if self.headers['METHOD'] == 'JSON':
+        if self.method == 'JSON':
             self.data = json.loads(body)
         else:
             self.data = {}
