@@ -51,8 +51,13 @@ import Cookie
 import base64
 import hmac
 import cPickle as pickle
+from itertools import chain
 
 from mongrel2 import Mongrel2Connection, to_bytes, to_unicode
+from dictshield.base import ShieldException
+
+
+
 import ujson as json
 
 ###
@@ -60,6 +65,10 @@ import ujson as json
 ###
 HTTP_METHODS = ['get', 'post', 'put', 'delete', 'head', 'options', 'trace', 'connect']
 
+class FourOhFourException(Exception):
+    pass
+
+
 HTTP_FORMAT = "HTTP/1.1 %(code)s %(status)s\r\n%(headers)s\r\n\r\n%(body)s"
 def http_response(body, code, status, headers):
     """Renders arguments into an HTTP response.
@@ -307,7 +316,12 @@ class MessageHandler(object):
             try:
                 if not hasattr(self, '_url_args') or self._url_args is None:
                     self._url_args = []
-                rendered = fun(*self._url_args)
+
+                if isinstance(self._url_args, dict):
+                    #if the value was optional and not included, filter it out so the functions default takes priority
+                    rendered = fun(**dict((k, v) for k,v in self._url_args.items() if v)) 
+                else:
+                    rendered = fun(*self._url_args)
                 if rendered is None:
                     logging.debug('Handler had no return value: %s' % fun)
                     return ''
@@ -539,15 +553,272 @@ class JSONMessageHandler(WebMessageHandler):
                                         self.message.path,
                                         self.message.remote_addr))
         return response
-        
-    
 
+
+class JsonSchemaMessageHandler(WebMessageHandler):
+    manifest = {}
+
+    @classmethod
+    def add_model(self, model):
+        self.manifest[model.__name__.lower()]  = model.for_jsonschema()
+
+    def get(self):
+        self.set_body(json.dumps(self.manifest.values()))
+        return self.render(status_code=200)
+
+    def render(self, status_code=None, **kwargs):
+        if status_code:
+            self.set_status(status_code)
+
+        self.convert_cookies()
+        self.headers['Content-Type'] = "application/schema+json"
+
+
+        response = http_response(self.body, status_code,
+                                 self.status_msg, self.headers)
+        
+        return response
+
+MULTIPLE_ITEM_SEP = ','
+
+class AutoAPIBase(JSONMessageHandler):
+    model = None
+    queries = None 
+
+    ###
+    ### configuring input and output formats
+    ###
+
+    def _get_shields_from_postbody(self):
+        """ Describes how our incoming data looks
+        """
+        items = json.loads(str(self.get_argument('data'))) #ujson doesn't take unicode
+        shields = [self.model(**item) for item in items]
+        return shields
+
+
+    def _create_response(self, updated, failed=[], created=[]):
+        """Passed a list of shields and the state they're in, and creates a response
+        """
+        status = []
+        status.extend([{'status':201, 'id':str(shield.id), 'href':self.uri_for_shield(shield)} for shield in created])
+        status.extend([{'status':200, 'id':str(shield.id), 'href':self.uri_for_shield(shield)} for shield in updated])
+        status.extend([{'status':400, 'id':str(shield.id), 'href':self.uri_for_shield(shield)} for shield in failed])
+
+        self.add_to_payload('data', [shield.to_json(encode=False) for shield in chain(created, updated, failed)])
+        self.add_to_payload('multistatus', status)
+
+        status_code = self._get_status_code(updated, failed, created)
+        
+        return self.render(status_code=status_code)
+
+
+    ###
+    ### -General Validation and private computation
+    ###
+
+    def _get_status_code(self, updated, failed, created=[]):
+        """Creates the status code we should be returning based on our successes and failures
+        """
+        kinds = reduce(lambda old, new: old + 1 if new else old, [created, updated, failed], 0)
+        if kinds > 1:
+            status_code = 207 #multistatus!
+        else:
+            if failed:
+                status_code = 400
+            elif created:
+                status_code = 201
+            else:
+                status_code = 200
+        return status_code
+
+    def _pre_alter_validation(self):
+        """ Creates the shield objcts and validates that they're in the right format
+        if they're not, adds the error list to the payload
+        """
+        shields = self._get_shields_from_postbody()
+        invalid = self._validate(shields)
+
+        if invalid:
+            errors = [{'status':422,
+                       'id':shield.id,
+                       'error':error,
+                       'href':self.uri_for_shield(shield)
+                       } for shield, error in invalid]
+            self.add_to_payload('multistatus', json.dumps(errors))
+        return shields, invalid
+
+    def _validate(self, shields):
+        """ seperates the list of items into valid and invalid shields
+        """
+        invalid = []
+        for shield in shields:
+            try:
+                shield.validate()
+            except ShieldException, e:
+                invalid.append((shield, e))
+        return invalid
+
+    def url_matches_body(self, item_ids, shields):
+        """ We want to make sure that if the request asks for a specific few resources,
+        Those resources and only those resources are in the body
+        """
+        if not item_ids: return True 
+        for item_id, shield in zip(item_ids, shields):
+            if item_id != str(shield.id): # enforce a good request
+                return False
+        return True
+
+    def uri_for_shield(self, shield):
+        return str(shield.id)
+
+    ###
+    ### HTTP methods
+    ###
     
+    def get(self, item_ids=""):
+        """Handles read - either with a filter (item_ids) or a total list
+        """
+        try:
+            shields = self.read([v for v in item_ids.split(MULTIPLE_ITEM_SEP) if v])
+        except FourOhFourException:
+            return self.render(status_code=404)
+        return self._create_response(shields)
+
+
+    def post(self, item_ids=""):
+        """ Handles create if item_ids is missing, else
+        updates the items.
+
+        Items should be represented as objects 
+        inside a list, pegged to the global object  -  the global object name defaults to data but can be changed
+        by overriding the _get_shields_from_postbody method
+        e.g.
+        { 'data' : [
+                    {
+                     'mysamplekey1':somesamplevalue,
+                     'mysamplekey2':somesamplevalue,
+                    },
+                    {
+                     'mysamplekey1':somesamplevalue,
+                     'mysamplekey2':somesamplevalue,
+                    },
+                   ]
+        }
+
+        This keeps the interface constant if you're passing a single item or a list of items.
+        We only want to deal with sequences!
+        """
+        shields, invalid = self._pre_alter_validation()
+        if invalid:
+            return self.render(status_code=400)
+        if item_ids == "":
+            created, updated, failed = self.create(shields)
+            return self._create_response(updated, failed, created)
+        else:
+            if not self.url_matches_body(item_ids.split(MULTIPLE_ITEM_SEP), shields):
+                #TODO: add error message so client knows why the request failed
+                return self.render(status_code=400)
+
+            successes, failures = self.update(shields)
+
+            return self._create_response(successes, failures)
+            
+    def put(self, item_ids):
+        """ Handles update for 1 or many items.
+        Take the postbody and convert it into a list of shields, and then confirm that matches
+        the item ids passed in.
+
+                Items should be represented as a object
+        inside a list,  in an object with the "data" key.
+        e.g.
+        { 'data' : [
+                    {
+                     'mysamplekey1':somesamplevalue,
+                     'mysamplekey2':somesamplevalue,
+                    },
+                    {
+                     'mysamplekey1':somesamplevalue,
+                     'mysamplekey2':somesamplevalue,
+                    },
+                   ]
+        }
+
+        This keeps the interface constant if you're passing a single item or a list of items.
+        We only want to deal with sequences!
+        """
+        shields, invalid = self._pre_alter_validation()
+        if invalid:
+            return self.render(status_code=400)
+        if not self.url_matches_body(item_ids.split(MULTIPLE_ITEM_SEP), shields):
+            #TODO: add error message so client knows why the request failed
+            return self.render(status_code=400)
+        successes, failures = self.update(shields)
+        return self._create_response(successes, failures)
+
+    def delete(self, item_ids):
+        """ Handles delete for 1 or many items. Since this doesn't take a postbody, and just
+        Item ids, pass those on directly to destroy
+        """
+        item_ids = item_ids.split(MULTIPLE_ITEM_SEP)
+        try:
+            successes, failures = self.destroy(item_ids)
+        except FourOhFourException:
+            return self.render(status_code=404)
+        status_code = self._get_status_code(successes, failures)
+
+        status = []
+        status.extend([{'status':200, 'id':i} for i in successes])
+        status.extend([{'status':400, 'id':i} for i in failures])
+        self.add_to_payload('multistatus', json.dumps(status))
+
+        return self.render(status_code=status_code)
+
+    ###
+    ### -CRUD operations
+    ### 
+
+    def read(self, include):
+        """Returns a list of shields in the db.
+        takes a list of object ids to include - if that's empty then include everything
+        """
+        #TODO:Figure out how we want to pagify this or somehow break it down from a single monster rv
+        list_of_data = self.queries.read(include)
+        if include and not list_of_data:
+            raise FourOhFourException
+        return [self.model(**data) for data in list_of_data]
+
+    def create(self, shields):
+        """Ment for adding items to the database and returns a list of successful creations, updates and failures
+        Such that: 
+        created, updated, failed = self.create(shields)
+        """
+        return self.queries.create(shields) #lists of status and post-save representation
+
+    def destroy(self, item_ids):
+        """ Removes the passed ids from the datastore and returns a list of success and failures
+        Such that:
+        success, failure = self.destroy(item_ids)
+        """
+        return self.queries.destroy(item_ids)
+
+    def update(self, shields):
+        """ updates the passed sheilds in the datastore and returns a list of success and failures
+        Such that:
+        successes, failures = self.update(shields)
+        """
+        return self.queries.update(shields)
+
+
+
+
+
 ###
 ### Application logic
 ###
 
 class Brubeck(object):
+
     def __init__(self, mongrel2_pair=None, handler_tuples=None, pool=None,
                  no_handler=None, base_handler=None, template_loader=None,
                  log_level=logging.INFO, login_url=None, db_conn=None,
@@ -688,8 +959,10 @@ class Brubeck(object):
             url_check = regex.match(message.path)
 
             if url_check:
-                # Default must be empty list - `None` will fail
-                url_args = url_check.groups()
+                # `None` will fail, so we have to use at least an empty list
+                # We should try to use named arguments first, and if they're not present
+                # fall back to positional arguments
+                url_args = url_check.groupdict() or url_check.groups() or []
 
                 if inspect.isclass(kallable):
                     # Handler classes must be instantiated
@@ -699,7 +972,11 @@ class Brubeck(object):
                     return handler
                 else:
                     # Can't instantiate a function
-                    handler = lambda: kallable(self, message, *url_args)
+                    if isinstance(url_args, dict):
+                        #if the value was optional and not included, filter it out so the functions default takes priority
+                        handler = lambda: kallable(self, message, **dict((k, v) for k,v in url_args.items() if v))
+                    else:
+                        handler = lambda: kallable(self, message, *url_args)
                     return handler
             
         if handler is None:
@@ -707,6 +984,19 @@ class Brubeck(object):
 
         return handler
 
+    def register_api(self, APIClass):
+        model, model_name = APIClass.model, APIClass.model.__name__.lower()
+
+        if not JsonSchemaMessageHandler.manifest:
+            manifest_pattern = "/manifest.json"
+            self.add_route_rule(manifest_pattern, JsonSchemaMessageHandler)
+        
+        pattern = "/" + model_name  + "/((?P<item_ids>[-\w\d%s]+)/|$)" % MULTIPLE_ITEM_SEP
+        self.add_route_rule(pattern, APIClass)
+        JsonSchemaMessageHandler.add_model(model)
+
+        
+
     ###
     ### Application running functions
     ###
