@@ -16,7 +16,7 @@ def load_mako_env(template_dir, *args, **kwargs):
             return None
     return loader
   
-class MakoRendering(object):
+class MakoRendering(WebMessageHandler):
     def render_template(self, template_file,
                         _status_code=WebMessageHandler._SUCCESS_CODE,
                         **context):
@@ -43,12 +43,13 @@ def load_jinja2_env(template_dir, *args, **kwargs):
     def loader():
         from jinja2 import Environment, FileSystemLoader
         if template_dir is not None:
-            return Environment(loader=FileSystemLoader(template_dir or '.'), *args, **kwargs)
+            return Environment(loader=FileSystemLoader(template_dir or '.'),
+                               *args, **kwargs)
         else:
             return None
     return loader
 
-class Jinja2Rendering():
+class Jinja2Rendering(WebMessageHandler):
     """Jinja2Rendering is a mixin for for loading a Jinja2 rendering
     environment.
 
@@ -89,7 +90,7 @@ def load_tornado_env(template_dir, *args, **kwargs):
             return None
     return loader
 
-class TornadoRendering():
+class TornadoRendering(WebMessageHandler):
     """TornadoRendering is a mixin for for loading a Tornado rendering
     environment.
 
